@@ -1,4 +1,4 @@
-/* $Id: demowindow.cc,v 1.33 2002/04/02 13:12:56 daniel Exp $ */
+/* $Id: demowindow.cc,v 1.34 2002/04/27 14:13:12 murrayc Exp $ */
 
 /* demowindow.cc
  *
@@ -146,11 +146,11 @@ void DemoWindow::on_treeview_row_activated(const Gtk::TreePath& path, Gtk::TreeV
     {
       const DemoColumns& columns = demo_columns();
 
-      if(type_slotDo *const pSlot = m_refTreeStore->get_value(row, columns.slot))
+      type_slotDo *const pSlot = m_refTreeStore->get_value(row, columns.slot);
+      if(pSlot && (m_pWindow_Example = (*pSlot)()))
       {
         m_refTreeStore->set_value(row, columns.italic, true);
 
-        m_pWindow_Example = (*pSlot)(); //call Slot::operator()
         m_pWindow_Example->signal_hide().connect(SigC::slot(*this, &DemoWindow::on_example_window_hide));
         m_pWindow_Example->show();
       }
