@@ -27,7 +27,7 @@ filesel::filesel():
     /* Connect the ok_button_ to file_ok_sel function */
     get_ok_button()->signal_clicked().connect(slot(*this, &filesel::file_ok_sel));
     /* Connect the cancel_button_ to hiding the window */
-    get_cancel_button()->signal_clicked().connect(SigC::slot(*this, &filesel::hide));
+    get_cancel_button()->signal_clicked().connect(SigC::slot(*this, &Gtk::Widget::hide));
 }
 
 
