@@ -79,34 +79,23 @@ namespace Gdk
 /* The *_Class implementation: */
 
 GType DragContext_Class::get_type()
+{
+  if(!gtype_) // create the GType if necessary
   {
-    if (!gtype_) //Create the GType if necessary.
-      {
-        //Make sure that the parent type has been created:
-        CppClassParent::CppObjectType::get_type();
+    // TODO: This is currently just optimized away, apparently with no harm.
+    // Is it actually needed?
+    // Make sure that the parent type has been created.
+    CppClassParent::CppObjectType::get_type();
 
-        //Create a derived C type.
-        //These init funcs will, in turn, call the init functions in the base C type.
-        GTypeInfo info =
-          {
-            sizeof(BaseClassType), /* class_size */
-            0, /* base_init */
-            0, /* base_finalize */
-            (GClassInitFunc) &class_init_function, /* class_init */
-            0, /* class_finalize */
-            0, /* class_data */
-            sizeof(BaseObjectType), /* instance_size */
-            0, /* n_preallocs */
-            0, /* instance_init */
-	    0 /* value_table */
-          };
+    // Create the wrapper type, with the same class/instance size as the base type.
+    register_derived_type(gdk_drag_context_get_type(), (GClassInitFunc) &class_init_function);
 
-        gtype_ = g_type_register_static(gdk_drag_context_get_type(), "gdkmm__DragContext", &info, GTypeFlags(0));
-
-  //Add derived versions of interfaces, if the C type implements any interfaces:
-      }
-    return gtype_;
+    // Add derived versions of interfaces, if the C type implements any interfaces:
   }
+
+  return gtype_;
+}
+
 void DragContext_Class::class_init_function(BaseClassType* klass)
 {
   CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
@@ -216,9 +205,9 @@ void DragContext::set_icon(const Glib::RefPtr<Gdk::Pixbuf>& pixbuf, int hot_x, i
   gtk_drag_set_icon_pixbuf(gobj(), Glib::unwrap(pixbuf), hot_x, hot_y);
 }
 
-void DragContext::set_icon(const Glib::ustring& stock_id, int hot_x, int hot_y)
+void DragContext::set_icon(const Gtk::StockID& stock_id, int hot_x, int hot_y)
 {
-  gtk_drag_set_icon_stock(gobj(), stock_id.c_str(), hot_x, hot_y);
+  gtk_drag_set_icon_stock(gobj(), (stock_id).get_c_str(), hot_x, hot_y);
 }
 
 void DragContext::set_icon()
