@@ -71,11 +71,18 @@ namespace Gdk
 /* The *_Class implementation: */
 
 GType Visual_Class::get_type()
+{
+  if(!gtype_) // create the GType if necessary
   {
-    gtype_ = gdk_visual_get_type(); //The actual struct definition is hidden, so we can not derive one.
-    return gtype_;
+    // The actual struct definition is hidden, so we can not derive one.
+    gtype_ = gdk_visual_get_type();
+
+    // Add derived versions of interfaces, if the C type implements any interfaces:
   }
 
+  return gtype_;
+}
+
 
 Glib::ObjectBase* Visual_Class::wrap_new(GObject* o)
 {
