@@ -2,11 +2,9 @@
 #ifndef _GLIBMM_USTRING_H
 #define _GLIBMM_USTRING_H
 
-/* $Id: ustring.h,v 1.23 2002/04/09 13:26:04 daniel Exp $ */
+/* $Id: ustring.h,v 1.30 2002/04/30 16:49:12 daniel Exp $ */
 
-/* ustring.h
- *
- * Copyright (C) 2002 The gtkmm Development Team
+/* Copyright (C) 2002 The gtkmm Development Team
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -24,6 +22,7 @@
  */
 
 #include <glib/gunicode.h>
+#include <glibmm/unicode.h>
 
 #include <iosfwd>
 #include <iterator>
@@ -36,17 +35,60 @@ GTKMM_USING_STD(string)
 GTKMM_USING_STD(istream)
 GTKMM_USING_STD(ostream)
 
+#ifdef GTKMM_HAVE_STD_ITERATOR_TRAITS
+GTKMM_USING_STD(iterator_traits)
+#else
+#include <cstddef> /* for ptrdiff_t */
+GTKMM_USING_STD(random_access_iterator_tag)
+#endif
+
 
 namespace Glib
 {
 
-class ustring;
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+#ifndef GTKMM_HAVE_STD_ITERATOR_TRAITS
 
-/** The iterator type of Glib::ustring.  Note this is not a random
- * access iterator but a bidirectional one, since all index operations
- * need to iterate over the UTF-8 data.  Use std::advance() to move to
- * a certain position.  However, all of the relational operators are
- * available:  ==  !=  <  >  <=  >=
+template <class T>
+struct IteratorTraits
+{
+  typedef typename T::iterator_category iterator_category;
+  typedef typename T::value_type        value_type;
+  typedef typename T::difference_type   difference_type;
+  typedef typename T::pointer           pointer;
+  typedef typename T::reference         reference;
+};
+
+template <class T>
+struct IteratorTraits<T*>
+{
+  typedef std::random_access_iterator_tag iterator_category;
+  typedef T                               value_type;
+  typedef ptrdiff_t                       difference_type;
+  typedef T*                              pointer;
+  typedef T&                              reference;
+};
+
+template <class T>
+struct IteratorTraits<const T*>
+{
+  typedef std::random_access_iterator_tag iterator_category;
+  typedef T                               value_type;
+  typedef ptrdiff_t                       difference_type;
+  typedef const T*                        pointer;
+  typedef const T&                        reference;
+};
+
+#endif /* GTKMM_HAVE_STD_ITERATOR_TRAITS */
+#endif /* DOXYGEN_SHOULD_SKIP_THIS */
+
+
+/** The iterator type of Glib::ustring.
+ * Note this is not a random access iterator but a bidirectional one,
+ * since all index operations need to iterate over the UTF-8 data.  Use
+ * std::advance() to move to a certain position.  However, all of the
+ * relational operators are available:
+ * <tt>==&nbsp;!=&nbsp;<&nbsp;>&nbsp;<=&nbsp;>=</tt>
  *
  * A writeable iterator isn't provided because:  The number of bytes of
  * the old UTF-8 character and the new one to write could be different.
@@ -74,10 +116,8 @@ public:
   inline ustring_Iterator<T> &     operator--();
   inline const ustring_Iterator<T> operator--(int);
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
   explicit inline ustring_Iterator(T pos);
   inline T base() const;
-#endif /* DOXYGEN_SHOULD_SKIP_THIS */
 
 private:
   T pos_;
@@ -97,28 +137,39 @@ private:
 gunichar get_unichar_from_std_iterator(std::string::const_iterator pos);
 
 
-/** Glib::ustring has much the same interface as std::string,
- * but contains UTF-8 strings.
- * Note that a normal 7-bit ASCII string is also a UTF-8 string, so,
- * if you want to, you can use this class without even thinking about UTF-8.
- * (Except that any character values must be in range 0 to 127,
- *  therefore you cannot use 8-bit encodings like ISO-8859-1.)
+/** Glib::ustring has much the same interface as std::string, but contains
+ * Unicode characters encoded as UTF-8.
+ *
+ * The standard character set ANSI_X3.4-1968&nbsp;-- more commonly known as
+ * ASCII&nbsp;-- is a subset of UTF-8.  So, if you want to, you can use
+ * Glib::ustring without even thinking about UTF-8.
+ *
+ * @note Whenever ASCII is mentioned in this manual, we mean the @em real ASCII
+ * (i.e. as defined in ANSI_X3.4-1968), which contains only 7-bit characters.
+ * Glib::ustring can @em not be used with ASCII-compatible extended 8-bit
+ * charsets like ISO-8859-1.  It's a good idea to avoid string literals
+ * containing non-ASCII characters (e.g. German umlauts) in source code.
  *
  * You can find a detailed UTF-8 and Unicode FAQ here:
  * http://www.cl.cam.ac.uk/~mgk25/unicode.html
  *
- * Also, this class has conversions to and from std::string,
- * so you can use a std::string instead of a Glib::ustring --
- * However, this will not work with multi-byte translations,
- * just as normal C char* code wouldn't.
+ * Glib::ustring has implicit type conversions to and from std::string.
+ * These conversions do @em not convert to/from the current locale (see
+ * Glib::locale_from_utf8() and Glib::locale_to_utf8() if you need that).  You
+ * can always use std::string instead of Glib::ustring&nbsp;-- however, using
+ * std::string with multi-byte characters is quite hard.  For instance,
+ * <tt>std::string::operator[]</tt> might return a byte in the middle of a
+ * character, and <tt>std::string::length()</tt> returns the number of bytes
+ * rather than characters.  So don't do that without a good reason.
  *
- * In a perfect world the C++ Standard Library would contain
- * a UTF-8 string, but it doesn't.  Note that std::wstring is not
- * a UTF-8 string class because it contains only fixed-length characters.
+ * In a perfect world the C++ Standard Library would contain a UTF-8 string
+ * class.  Unfortunately, the C++ standard doesn't mention UTF-8 at all.  Note
+ * that std::wstring is not a UTF-8 string class because it contains only
+ * fixed-width characters (where width could be 32, 16, or even 8 bits).
  *
- * Glib::ustring does not inherit from std::string, because std::string
- * was intended to be a final class.  For instance, it does not have
- * a virtual destructor.
+ * Glib::ustring does not inherit from std::string, because std::string was
+ * intended to be a final class.  For instance, it does not have a virtual
+ * destructor.
  */
 class ustring
 {
@@ -132,29 +183,52 @@ public:
 
   typedef ustring_Iterator<std::string::iterator>       iterator;
   typedef ustring_Iterator<std::string::const_iterator> const_iterator;
+
+#ifndef GTKMM_HAVE_SUN_REVERSE_ITERATOR
+
   typedef std::reverse_iterator<iterator>               reverse_iterator;
   typedef std::reverse_iterator<const_iterator>         const_reverse_iterator;
 
+#else
+
+  typedef std::reverse_iterator<iterator,
+                                iterator::iterator_category,
+                                iterator::value_type,
+                                iterator::reference,
+                                iterator::pointer,
+                                iterator::difference_type> reverse_iterator;
+  typedef std::reverse_iterator<const_iterator,
+                                const_iterator::iterator_category,
+                                const_iterator::value_type,
+                                const_iterator::reference,
+                                const_iterator::pointer,
+                                const_iterator::difference_type> const_reverse_iterator;
+
+#endif /* GTKMM_HAVE_SUN_REVERSE_ITERATOR */
+
   static const size_type npos = std::string::npos;
 
   ustring();
-  ustring(const ustring& src);
+  ~ustring();
+
+  ustring(const ustring& other);
+  ustring& operator=(const ustring& other);
+
+  void swap(ustring& other);
+
+//! @{ @name Create new strings.
+
+  ustring(const std::string& src);
   ustring(const ustring& src, size_type i, size_type n=npos);
   ustring(const char* src, size_type n);
   ustring(const char* src);
   ustring(size_type n, gunichar uc);
   ustring(size_type n, char c);
-  template <class In> inline
-    ustring(In pbegin, In pend);
+  template <class In> ustring(In pbegin, In pend);
 
-  ustring(const std::string& src);
-  explicit ustring(const gunichar* src, size_type n=npos);
+//! @}
+//! @{ @name Assign new contents.
 
-  ~ustring();
-
-  void swap(ustring& other);
-
-  ustring& operator=(const ustring& src);
   ustring& operator=(const std::string& src);
   ustring& operator=(const char* src);
   ustring& operator=(gunichar uc);
@@ -166,8 +240,10 @@ public:
   ustring& assign(const char* src);
   ustring& assign(size_type n, gunichar uc);
   ustring& assign(size_type n, char c);
-  template <class In> inline
-    ustring& assign(In pbegin, In pend);
+  template <class In> ustring& assign(In pbegin, In pend);
+
+//! @}
+//! @{ @name Append to the string.
 
   ustring& operator+=(const ustring& src);
   ustring& operator+=(const char* src);
@@ -182,8 +258,10 @@ public:
   ustring& append(const char* src);
   ustring& append(size_type n, gunichar uc);
   ustring& append(size_type n, char c);
-  template <class In> inline
-    ustring& append(In pbegin, In pend);
+  template <class In> ustring& append(In pbegin, In pend);
+
+//! @}
+//! @{ @name Insert into the string.
 
   ustring& insert(size_type i, const ustring& src);
   ustring& insert(size_type i, const ustring& src, size_type i2, size_type n);
@@ -196,8 +274,10 @@ public:
   iterator insert(iterator p, char c);
   void     insert(iterator p, size_type n, gunichar uc);
   void     insert(iterator p, size_type n, char c);
-  template <class In> inline
-    void insert(iterator p, In pbegin, In pend);
+  template <class In> void insert(iterator p, In pbegin, In pend);
+
+//! @}
+//! @{ @name Replace sub-strings.
 
   ustring& replace(size_type i, size_type n, const ustring& src);
   ustring& replace(size_type i, size_type n, const ustring& src, size_type i2, size_type n2);
@@ -211,14 +291,19 @@ public:
   ustring& replace(iterator pbegin, iterator pend, const char* src);
   ustring& replace(iterator pbegin, iterator pend, size_type n, gunichar uc);
   ustring& replace(iterator pbegin, iterator pend, size_type n, char c);
-  template <class In> inline
-    ustring& replace(iterator pbegin, iterator pend, In pbegin2, In pend2);
+  template <class In> ustring& replace(iterator pbegin, iterator pend, In pbegin2, In pend2);
+
+//! @}
+//! @{ @name Erase sub-strings.
 
   ustring& erase(size_type i, size_type n=npos);
   ustring& erase();
   iterator erase(iterator p);
   iterator erase(iterator pbegin, iterator pend);
 
+//! @}
+//! @{ @name Compare and collate.
+
   int compare(const ustring& rhs) const;
   int compare(const char* rhs)    const;
   int compare(size_type i, size_type n, const ustring& rhs) const;
@@ -226,12 +311,24 @@ public:
   int compare(size_type i, size_type n, const char* rhs, size_type n2) const;
   int compare(size_type i, size_type n, const char* rhs) const;
 
-  /** @note No reference return; use replace() to write characters. */
+  /*! Create a unique sorting key for the UTF-8 string. */
+  std::string collate_key() const;
+  std::string casefold_collate_key() const;
+
+//! @}
+//! @{ @name Extract characters and sub-strings.
+
+  /*! No reference return; use replace() to write characters. */
   value_type operator[](size_type i) const;
+
+  /*! No reference return; use replace() to write characters. @throw std::out_of_range */
   value_type at(size_type i) const;
 
   inline ustring substr(size_type i=0, size_type n=npos) const;
 
+//! @}
+//! @{ @name Access a sequence of characters.
+
   iterator begin();
   iterator end();
   const_iterator begin() const;
@@ -241,6 +338,9 @@ public:
   const_reverse_iterator rbegin() const;
   const_reverse_iterator rend()   const;
 
+//! @}
+//! @{ @name Find sub-strings.
+
   size_type find(const ustring& str, size_type i=0) const;
   size_type find(const char* str, size_type i, size_type n) const;
   size_type find(const char* str, size_type i=0) const;
@@ -253,6 +353,9 @@ public:
   size_type rfind(gunichar uc, size_type i=npos) const;
   size_type rfind(char c, size_type i=npos) const;
 
+//! @}
+//! @{ @name Match against a set of characters.
+
   size_type find_first_of(const ustring& match, size_type i=0) const;
   size_type find_first_of(const char* match, size_type i, size_type n) const;
   size_type find_first_of(const char* match, size_type i=0) const;
@@ -277,18 +380,30 @@ public:
   size_type find_last_not_of(gunichar uc, size_type i=npos) const;
   size_type find_last_not_of(char c, size_type i=npos) const;
 
+//! @}
+//! @{ @name Retrieve the string's size.
+
   bool      empty()  const;
   size_type size()   const;
   size_type length() const;
   size_type bytes()  const;
 
-  size_type capacity() const;
-  size_type max_size() const;
+//! @}
+//! @{ @name Change the string's size.
 
   void resize(size_type n, gunichar uc);
   void resize(size_type n, char c='\0');
+
+//! @}
+//! @{ @name Control the allocated memory.
+
+  size_type capacity() const;
+  size_type max_size() const;
   void reserve(size_type n=0);
 
+//! @}
+//! @{ @name Get a per-byte representation of the string.
+
   inline operator std::string() const; // e.g. std::string str = ustring();
   inline const std::string& raw() const;
 
@@ -296,15 +411,18 @@ public:
   const char* data()  const;
   const char* c_str() const;
 
-  size_type copy(char*     p, size_type n, size_type i=0) const;
-  size_type copy(gunichar* p, size_type n, size_type i=0) const;
+  /*! @return Number of copied @em bytes, not characters. */
+  size_type copy(char* dest, size_type n, size_type i=0) const;
 
-  /** Check whether the string contains valid UTF-8 data. */
+//! @}
+//! @{ @name UTF-8 utilities.
+
+  /*! Check whether the string is valid UTF-8. */
   bool validate() const;
   bool validate(iterator& first_invalid);
   bool validate(const_iterator& first_invalid) const;
 
-  /** Check whether the string is plain 7-bit ASCII.
+  /*! Check whether the string is plain 7-bit ASCII. @par
    * Unlike any other ustring method, is_ascii() is safe to use on invalid
    * UTF-8 strings.  If the string isn't valid UTF-8, it cannot be valid
    * ASCII either, therefore is_ascii() will just return @c false then.
@@ -312,24 +430,68 @@ public:
    */
   bool is_ascii() const;
 
-  /** 'Normalize' the Unicode character values of the string. */
-  ustring normalize(GNormalizeMode mode = G_NORMALIZE_DEFAULT_COMPOSE) const;
+  /*! "Normalize" the Unicode character representation of the string. */
+  ustring normalize(NormalizeMode mode = NORMALIZE_DEFAULT_COMPOSE) const;
+
+//! @}
+//! @{ @name Character case conversion.
 
   ustring uppercase() const;
   ustring lowercase() const;
   ustring casefold()  const;
 
-  /** Create a unique sorting key for the UTF-8 string. */
-  std::string collate_key() const;
-  std::string casefold_collate_key() const;
+//! @}
 
 private:
-  template <class Iterator>
-    struct IteratorConvert_;
+
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+
+#ifdef GTKMM_HAVE_STD_ITERATOR_TRAITS
+  template <class In, class ValueType = typename std::iterator_traits<In>::value_type>
+#else
+  template <class In, class ValueType = typename Glib::IteratorTraits<In>::value_type>
+#endif
+  struct SequenceToString;
+
+#endif /* DOXYGEN_SHOULD_SKIP_THIS */
 
   std::string string_;
 };
 
+
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+
+template <class In, class ValueType>
+struct ustring::SequenceToString
+{};
+
+template <class In>
+struct ustring::SequenceToString<In,char> : public std::string
+{
+  SequenceToString(In pbegin, In pend);
+};
+
+template <class In>
+struct ustring::SequenceToString<In,gunichar> : public std::string
+{
+  SequenceToString(In pbegin, In pend);
+};
+
+template <>
+struct ustring::SequenceToString<Glib::ustring::iterator,gunichar> : public std::string
+{
+  SequenceToString(Glib::ustring::iterator pbegin, Glib::ustring::iterator pend);
+};
+
+template <>
+struct ustring::SequenceToString<Glib::ustring::const_iterator,gunichar> : public std::string
+{
+  SequenceToString(Glib::ustring::const_iterator pbegin, Glib::ustring::const_iterator pend);
+};
+
+#endif /* DOXYGEN_SHOULD_SKIP_THIS */
+
+
 /** Stream input operator.
  * @relates Glib::ustring
  * @throw Glib::ConvertError
@@ -350,8 +512,6 @@ std::ostream& operator<<(std::ostream& os, const Glib::ustring& utf8_string);
 
 /**** Glib::ustring_Iterator<> *********************************************/
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
-
 template <class T> inline
 ustring_Iterator<T>::ustring_Iterator(T pos)
 :
@@ -364,8 +524,6 @@ T ustring_Iterator<T>::base() const
   return pos_;
 }
 
-#endif /* DOXYGEN_SHOULD_SKIP_THIS */
-
 template <class T> inline
 ustring_Iterator<T>::ustring_Iterator()
 :
@@ -465,74 +623,72 @@ bool operator>=(const ustring_Iterator<T>& lhs, const ustring_Iterator<T>& rhs)
 }
 
 
-/**** Glib::ustring::IteratorConvert_<> ************************************/
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
 
-template <class Iterator>
-struct ustring::IteratorConvert_
-{
-  static inline Iterator
-  get_base(Iterator p) { return p; }
-};
+/**** Glib::ustring::SequenceToString **************************************/
 
-template <>
-struct ustring::IteratorConvert_<Glib::ustring::iterator>
-{
-  static inline std::string::iterator
-  get_base(Glib::ustring::iterator p) { return p.base(); }
-};
+template <class In>
+ustring::SequenceToString<In,char>::SequenceToString(In pbegin, In pend)
+:
+  std::string(pbegin, pend)
+{}
 
-template <>
-struct ustring::IteratorConvert_<Glib::ustring::const_iterator>
+template <class In>
+ustring::SequenceToString<In,gunichar>::SequenceToString(In pbegin, In pend)
 {
-  static inline std::string::const_iterator
-  get_base(Glib::ustring::const_iterator p) { return p.base(); }
-};
+  char utf8_buf[6]; // stores a single UTF-8 character
+
+  for(; pbegin != pend; ++pbegin)
+  {
+    const std::string::size_type utf8_len = g_unichar_to_utf8(*pbegin, utf8_buf);
+    this->append(utf8_buf, utf8_len);
+  }
+}
+
+#endif /* DOXYGEN_SHOULD_SKIP_THIS */
 
 
 /**** Glib::ustring ********************************************************/
 
-template <class In> inline
+template <class In>
 ustring::ustring(In pbegin, In pend)
 :
-  string_ (IteratorConvert_<In>::get_base(pbegin),
-           IteratorConvert_<In>::get_base(pend))
+  string_ (Glib::ustring::SequenceToString<In>(pbegin, pend))
 {}
 
-template <class In> inline
+template <class In>
 ustring& ustring::assign(In pbegin, In pend)
 {
-  typedef IteratorConvert_<In> Conv;
-  string_.assign(Conv::get_base(pbegin), Conv::get_base(pend));
+  Glib::ustring::SequenceToString<In> temp_string (pbegin, pend);
+  string_.swap(temp_string); // constant-time operation
   return *this;
 }
 
-template <class In> inline
+template <class In>
 ustring& ustring::append(In pbegin, In pend)
 {
-  typedef IteratorConvert_<In> Conv;
-  string_.append(Conv::get_base(pbegin), Conv::get_base(pend));
+  string_.append(Glib::ustring::SequenceToString<In>(pbegin, pend));
   return *this;
 }
 
-template <class In> inline
+template <class In>
 void ustring::insert(ustring::iterator p, In pbegin, In pend)
 {
-  typedef IteratorConvert_<In> Conv;
-  string_.insert(p.base(), Conv::get_base(pbegin), Conv::get_base(pend));
+  string_.insert(p.base(), Glib::ustring::SequenceToString<In>(pbegin, pend));
 }
 
-template <class In> inline
+template <class In>
 ustring& ustring::replace(ustring::iterator pbegin, ustring::iterator pend, In pbegin2, In pend2)
 {
-  typedef IteratorConvert_<In> Conv;
   string_.replace(
       pbegin.base(), pend.base(),
-      Conv::get_base(pbegin2), Conv::get_base(pend2));
+      Glib::ustring::SequenceToString<In>(pbegin2, pend2));
   return *this;
 }
 
-// substr() and operator std::string() are inline, so that
-// the compiler has a fair chance to optimize the copy ctor away.
+// The ustring methods substr() and operator std::string() are inline,
+// so that the compiler has a fair chance to optimize the copy ctor away.
+
 inline
 ustring ustring::substr(ustring::size_type i, ustring::size_type n) const
 {
