@@ -3,7 +3,7 @@
 #include <gtkmm/private/bin_p.h>
 
 // -*- c++ -*-
-/* $Id: bin.ccg,v 1.12 2002/02/15 13:57:22 murrayc Exp $ */
+/* $Id: bin.ccg,v 1.13 2002/04/21 23:37:05 murrayc Exp $ */
 
 /* 
  *
@@ -43,8 +43,8 @@ Bin::remove()
 }
 
 void 
-Bin::add_label(const Glib::ustring& str, bool mnemonic = false,
-	             double x_align = 0.5, double y_align = 0.5)
+Bin::add_label(const Glib::ustring& str, bool mnemonic /* = false */,
+	             double x_align /* = 0.5 */, double y_align /* = 0.5 */)
 {
   Label* label = manage(new Label(str, mnemonic));
   label->set_alignment(x_align, y_align);
@@ -70,7 +70,7 @@ void
 Bin::add_pixlabel (const Glib::RefPtr<Gdk::Pixmap>& pixmap,
 		   const Glib::RefPtr<Gdk::Bitmap>& mask,
 		   const Glib::ustring& str,
-		   double x_align = 0.5, double y_align = 0.5)
+		   double x_align /* = 0.5 */, double y_align /* = 0.5 */)
 {
   //Create Pixmap and Label widgets:
   Image* pmap = manage(new Image(pixmap, mask));
@@ -90,7 +90,7 @@ Bin::add_pixlabel (const Glib::RefPtr<Gdk::Pixmap>& pixmap,
 void
 Bin::add_pixlabel(const Glib::ustring& pixfile,
 		  const Glib::ustring& str,
-		  double x_align = 0.5, double y_align = 0.5)
+		  double x_align /* = 0.5 */, double y_align /* = 0.5 */)
 {
   //Create Pixmap and Label widgets:
   Image* pmap = manage(new Image(pixfile));
@@ -133,34 +133,23 @@ namespace Gtk
 /* The *_Class implementation: */
 
 GType Bin_Class::get_type()
+{
+  if(!gtype_) // create the GType if necessary
   {
-    if (!gtype_) //Create the GType if necessary.
-      {
-        //Make sure that the parent type has been created:
-        CppClassParent::CppObjectType::get_type();
+    // TODO: This is currently just optimized away, apparently with no harm.
+    // Is it actually needed?
+    // Make sure that the parent type has been created.
+    CppClassParent::CppObjectType::get_type();
 
-        //Create a derived C type.
-        //These init funcs will, in turn, call the init functions in the base C type.
-        GTypeInfo info =
-          {
-            sizeof(BaseClassType), /* class_size */
-            0, /* base_init */
-            0, /* base_finalize */
-            (GClassInitFunc) &class_init_function, /* class_init */
-            0, /* class_finalize */
-            0, /* class_data */
-            sizeof(BaseObjectType), /* instance_size */
-            0, /* n_preallocs */
-            0, /* instance_init */
-	    0 /* value_table */
-          };
+    // Create the wrapper type, with the same class/instance size as the base type.
+    register_derived_type(gtk_bin_get_type(), (GClassInitFunc) &class_init_function);
 
-        gtype_ = g_type_register_static(gtk_bin_get_type(), "gtkmm__Bin", &info, GTypeFlags(0));
-
-  //Add derived versions of interfaces, if the C type implements any interfaces:
-      }
-    return gtype_;
+    // Add derived versions of interfaces, if the C type implements any interfaces:
   }
+
+  return gtype_;
+}
+
 void Bin_Class::class_init_function(BaseClassType* klass)
 {
   CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
