@@ -3,7 +3,7 @@
 #include <gtkmm/private/button_p.h>
 
 // -*- c++ -*-
-/* $Id: button.ccg,v 1.13 2002/04/02 13:25:13 daniel Exp $ */
+/* $Id: button.ccg,v 1.15 2002/04/26 18:25:56 murrayc Exp $ */
 
 /* 
  *
@@ -40,16 +40,10 @@ Button::Button(const Glib::ustring& label, bool mnemonic /* = false */)
       get_type(), "label", label.c_str(), "use_underline", gboolean(mnemonic), (char*)0))
 {}
 
-Button::Button(const StockID& id)
+Button::Button(const StockID& stock_id)
 :
   Gtk::Bin((GtkBin*) g_object_new(
-      get_type(), "use_stock", 1, "label", id.get_string().c_str(), (char*)0))
-{}
-
-Button::Button(const Stock::BuiltinStockID& id)
-:
-  Gtk::Bin((GtkBin*) g_object_new(
-      get_type(), "use_stock", 1, "label", id.id, (char*)0))
+      get_type(), "use_stock", 1, "label", stock_id.get_c_str(), (char*)0))
 {}
 
 } // namespace Gtk
@@ -125,34 +119,23 @@ namespace Gtk
 /* The *_Class implementation: */
 
 GType Button_Class::get_type()
+{
+  if(!gtype_) // create the GType if necessary
   {
-    if (!gtype_) //Create the GType if necessary.
-      {
-        //Make sure that the parent type has been created:
-        CppClassParent::CppObjectType::get_type();
+    // TODO: This is currently just optimized away, apparently with no harm.
+    // Is it actually needed?
+    // Make sure that the parent type has been created.
+    CppClassParent::CppObjectType::get_type();
 
-        //Create a derived C type.
-        //These init funcs will, in turn, call the init functions in the base C type.
-        GTypeInfo info =
-          {
-            sizeof(BaseClassType), /* class_size */
-            0, /* base_init */
-            0, /* base_finalize */
-            (GClassInitFunc) &class_init_function, /* class_init */
-            0, /* class_finalize */
-            0, /* class_data */
-            sizeof(BaseObjectType), /* instance_size */
-            0, /* n_preallocs */
-            0, /* instance_init */
-	    0 /* value_table */
-          };
+    // Create the wrapper type, with the same class/instance size as the base type.
+    register_derived_type(gtk_button_get_type(), (GClassInitFunc) &class_init_function);
 
-        gtype_ = g_type_register_static(gtk_button_get_type(), "gtkmm__Button", &info, GTypeFlags(0));
-
-  //Add derived versions of interfaces, if the C type implements any interfaces:
-      }
-    return gtype_;
+    // Add derived versions of interfaces, if the C type implements any interfaces:
   }
+
+  return gtype_;
+}
+
 void Button_Class::class_init_function(BaseClassType* klass)
 {
   CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
