@@ -1,5 +1,5 @@
 // -*- c++ -*-
-/* $Id: clipboard.cc,v 1.1 2002/02/25 12:27:37 murrayc Exp $ */
+/* $Id: clipboard.cc,v 1.2 2002/04/22 00:05:38 murrayc Exp $ */
 
 /* stockid.cc
  *
@@ -195,7 +195,7 @@ bool Clipboard::set(const GtkTargetEntry* targets, guint n_targets, SlotGet slot
   //Create a signal proxy. A pointer to this will be passed through the callback's data parameter.
   SignalProxy_GetClear* pSignalProxy = new SignalProxy_GetClear(slot_get, slot_clear);
 
-  gtk_clipboard_set_with_data(gobj(), targets, n_targets,
+  return gtk_clipboard_set_with_data(gobj(), targets, n_targets,
     &SignalProxy_GetClear::gtk_callback_get, &SignalProxy_GetClear::gtk_callback_clear, (gpointer)pSignalProxy);
 }
 
