@@ -3,7 +3,7 @@
 #include <gtkmm/private/container_p.h>
 
 // -*- c++ -*-
-/* $Id: container.ccg,v 1.14 2002/03/21 12:44:01 murrayc Exp $ */
+/* $Id: container.ccg,v 1.15 2002/04/19 22:05:03 daniel Exp $ */
 
 /* 
  *
@@ -25,24 +25,44 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <sigc++/method_slot.h>
 #include <gtkmm/adjustment.h>
 #include <gtk/gtkcontainer.h>
 
+
+namespace
+{
+extern "C"
+{
+
+void container_foreach_callback(GtkWidget* widget_gobj, void* data)
+{
+  try
+  {
+    Gtk::Container::ForeachSlot& slot = *static_cast<Gtk::Container::ForeachSlot*>(data);
+    Gtk::Widget *const widget = Glib::wrap(widget_gobj);
+
+    g_return_if_fail(widget != 0);
+
+    slot(*widget);
+  }
+  catch(...)
+  {
+    Glib::exception_handlers_invoke();
+  }
+}
+
+} // extern "C"
+} // anonymous namespace
+
+
 namespace Gtk
 {
 
-void foreach_callback_(GtkWidget* w, gpointer data)
+void Container::foreach(const Container::ForeachSlot& slot)
 {
-  Gtk::Container::ForeachCallback *s=(Gtk::Container::ForeachCallback*)data;
-  if (!w) return;
-  (*s)(*Glib::wrap(w));
-}
-
-void Container::foreach(const ForeachCallback& s)
-{
-   gtk_container_foreach(gobj(),
-                         &foreach_callback_,
-                         (void*)(&s));
+  ForeachSlot slot_copy (slot);
+  gtk_container_foreach(gobj(), &container_foreach_callback, &slot_copy);
 }
 
 void  Container::add(Widget& widget)
@@ -50,7 +70,6 @@ void  Container::add(Widget& widget)
   gtk_container_add(gobj(), widget.gobj());
 }
 
-
 bool Container::has_focus_chain() const
 {
   return gtk_container_get_focus_chain(const_cast<GtkContainer*>(gobj()), 0);
@@ -64,21 +83,20 @@ Glib::ListHandle<Widget*> Container::get_focus_chain()
   return Glib::ListHandle<Widget*>(list, Glib::OWNERSHIP_SHALLOW);
 }
 
-void Container::show_all_children(bool recursive /* = true */)
+void Container::show_all_children(bool recursive)
 {
-  typedef std::list<Widget*> typeListChildren;
-  typeListChildren listChildren = get_children();
+  // Plain C version if this turns out to be performance-critical:
+  //GtkCallback callback = (GtkCallback) ((recursive) ? &gtk_widget_show_all : &gtk_widget_show);
+  //gtk_container_foreach(gobj(), callback, 0);
 
-  for(typeListChildren::iterator iter = listChildren.begin(); iter != listChildren.end(); iter++)
-  {
-    if(recursive)
-      (*iter)->show_all();
-    else
-      (*iter)->show();
-  }
+  // This could also be done with get_children() and an explicit loop,
+  // if any problems should arise.  In gtkmm-1.2, foreach() is known to
+  // cause segfaults eventually, but it seems to work now.
+
+  foreach(SigC::slot((recursive) ? &Widget::show_all : &Widget::show));
 }
 
-} //namespace Gtk
+} // namespace Gtk
 
 
 namespace
@@ -190,34 +208,23 @@ namespace Gtk
 /* The *_Class implementation: */
 
 GType Container_Class::get_type()
+{
+  if(!gtype_) // create the GType if necessary
   {
-    if (!gtype_) //Create the GType if necessary.
-      {
-        //Make sure that the parent type has been created:
-        CppClassParent::CppObjectType::get_type();
+    // TODO: This is currently just optimized away, apparently with no harm.
+    // Is it actually needed?
+    // Make sure that the parent type has been created.
+    CppClassParent::CppObjectType::get_type();
 
-        //Create a derived C type.
-        //These init funcs will, in turn, call the init functions in the base C type.
-        GTypeInfo info =
-          {
-            sizeof(BaseClassType), /* class_size */
-            0, /* base_init */
-            0, /* base_finalize */
-            (GClassInitFunc) &class_init_function, /* class_init */
-            0, /* class_finalize */
-            0, /* class_data */
-            sizeof(BaseObjectType), /* instance_size */
-            0, /* n_preallocs */
-            0, /* instance_init */
-	    0 /* value_table */
-          };
+    // Create the wrapper type, with the same class/instance size as the base type.
+    register_derived_type(gtk_container_get_type(), (GClassInitFunc) &class_init_function);
 
-        gtype_ = g_type_register_static(gtk_container_get_type(), "gtkmm__Container", &info, GTypeFlags(0));
-
-  //Add derived versions of interfaces, if the C type implements any interfaces:
-      }
-    return gtype_;
+    // Add derived versions of interfaces, if the C type implements any interfaces:
   }
+
+  return gtype_;
+}
+
 void Container_Class::class_init_function(BaseClassType* klass)
 {
   CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
@@ -555,11 +562,6 @@ void Container::check_resize()
   gtk_container_check_resize(gobj());
 }
 
-void Container::foreach_(GtkCallback callback, gpointer data)
-{
-  gtk_container_foreach(gobj(), callback, data);
-}
-
 Glib::ListHandle<Widget*> Container::get_children()
 {
   return Glib::ListHandle<Widget*>(gtk_container_get_children(gobj()), Glib::OWNERSHIP_SHALLOW);
