@@ -4,11 +4,9 @@
 
 #include <gtk/gtktypebuiltins.h>
 // -*- c++ -*-
-/* $Id: dialog.ccg,v 1.6 2002/03/09 18:10:46 murrayc Exp $ */
+/* $Id: dialog.ccg,v 1.7 2002/04/23 16:33:35 daniel Exp $ */
 
-/* 
- *
- * Copyright 1998-1999 The Gtk-- Development Team
+/* Copyright 1998-1999 The Gtk-- Development Team
  * Copyright 2001      Free Software Foundation
  *
  * This library is free software; you can redistribute it and/or
@@ -31,15 +29,17 @@
 namespace Gtk
 {
 
-Dialog::Dialog(const Glib::ustring& title, Gtk::Window& parent, bool modal /* = false */, bool use_separator /* = true */)
-: Gtk::Window((GtkWindow*)( g_object_new( get_type() , "title", title.c_str(), 0 ) ))
+Dialog::Dialog(const Glib::ustring& title, Gtk::Window& parent, bool modal, bool use_separator)
+:
+  Gtk::Window((GtkWindow*) g_object_new(get_type(), "title", title.c_str(), (char*)0))
 {
   construct_(modal, use_separator);
   set_transient_for(parent);
 }
 
-Dialog::Dialog(const Glib::ustring& title, bool modal /* = false */, bool use_separator /* = true */)
-: Gtk::Window((GtkWindow*)( g_object_new( get_type() , "title", title.c_str(), 0 ) ))
+Dialog::Dialog(const Glib::ustring& title, bool modal, bool use_separator)
+:
+  Gtk::Window((GtkWindow*) g_object_new(get_type(), "title", title.c_str(), (char*)0))
 {
   construct_(modal, use_separator);
 }
@@ -47,11 +47,10 @@ Dialog::Dialog(const Glib::ustring& title, bool modal /* = false */, bool use_se
 void Dialog::construct_(bool modal, bool use_separator)
 {
   set_modal(modal);
-  set_has_separator(modal);
+  set_has_separator(use_separator);
 }
 
-
-} /* namespace Gtk */
+} // namespace Gtk
 
 
 namespace
@@ -117,34 +116,23 @@ namespace Gtk
 /* The *_Class implementation: */
 
 GType Dialog_Class::get_type()
+{
+  if(!gtype_) // create the GType if necessary
   {
-    if (!gtype_) //Create the GType if necessary.
-      {
-        //Make sure that the parent type has been created:
-        CppClassParent::CppObjectType::get_type();
+    // TODO: This is currently just optimized away, apparently with no harm.
+    // Is it actually needed?
+    // Make sure that the parent type has been created.
+    CppClassParent::CppObjectType::get_type();
 
-        //Create a derived C type.
-        //These init funcs will, in turn, call the init functions in the base C type.
-        GTypeInfo info =
-          {
-            sizeof(BaseClassType), /* class_size */
-            0, /* base_init */
-            0, /* base_finalize */
-            (GClassInitFunc) &class_init_function, /* class_init */
-            0, /* class_finalize */
-            0, /* class_data */
-            sizeof(BaseObjectType), /* instance_size */
-            0, /* n_preallocs */
-            0, /* instance_init */
-	    0 /* value_table */
-          };
+    // Create the wrapper type, with the same class/instance size as the base type.
+    register_derived_type(gtk_dialog_get_type(), (GClassInitFunc) &class_init_function);
 
-        gtype_ = g_type_register_static(gtk_dialog_get_type(), "gtkmm__Dialog", &info, GTypeFlags(0));
-
-  //Add derived versions of interfaces, if the C type implements any interfaces:
-      }
-    return gtype_;
+    // Add derived versions of interfaces, if the C type implements any interfaces:
   }
+
+  return gtype_;
+}
+
 void Dialog_Class::class_init_function(BaseClassType* klass)
 {
   CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
@@ -254,6 +242,11 @@ Widget* Dialog::add_button(const Glib::ustring& button_text, int response_id)
   return Glib::wrap(gtk_dialog_add_button(gobj(), button_text.c_str(), response_id));
 }
 
+Widget* Dialog::add_button(const Gtk::StockID& stock_id, int response_id)
+{
+  return Glib::wrap(gtk_dialog_add_button(gobj(), (stock_id).get_c_str(), response_id));
+}
+
 void Dialog::set_response_sensitive(int response_id, bool setting)
 {
   gtk_dialog_set_response_sensitive(gobj(), response_id, static_cast<int>(setting));
@@ -294,14 +287,14 @@ const VBox* Dialog::get_vbox() const
   return Glib::wrap((GtkVBox*)(gobj()->vbox));
 }
 
-HBox* Dialog::get_action_area()
+HButtonBox* Dialog::get_action_area()
 {
-  return Glib::wrap((GtkHBox*)(gobj()->action_area));
+  return Glib::wrap((GtkHButtonBox*)(gobj()->action_area));
 }
 
-const HBox* Dialog::get_action_area() const
+const HButtonBox* Dialog::get_action_area() const
 {
-  return Glib::wrap((GtkHBox*)(gobj()->action_area));
+  return Glib::wrap((GtkHButtonBox*)(gobj()->action_area));
 }
 
 
