@@ -3,7 +3,7 @@
 #include <gtkmm/private/liststore_p.h>
 
 // -*- c++ -*-
-/* $Id: liststore.ccg,v 1.11 2002/04/08 17:20:02 daniel Exp $ */
+/* $Id: liststore.ccg,v 1.12 2002/04/16 13:01:52 murrayc Exp $ */
 
 /* 
  *
@@ -27,6 +27,7 @@
 
 #include <gtkmm/textmark.h>
 #include <glibmm/value.h>
+#include <gtk/gtkliststore.h>
 
 namespace Gtk
 {
@@ -42,10 +43,10 @@ TreeIter ListStore::erase(const TreeIter& iter)
 {
   g_assert(iter.get_gobject_if_not_end() != 0);
 
-  TreeIter next (iter);
+  TreeIter next(iter);
   ++next;
 
-  GtkTreeIter tmp = *iter.gobj();
+  GtkTreeIter tmp = *(iter.gobj());
   gtk_list_store_remove(gobj(), &tmp);
 
   return next;
@@ -57,10 +58,10 @@ TreeIter ListStore::insert(const TreeIter& iter)
 
   gtk_list_store_insert_before(
       gobj(), new_pos.gobj(),
-      const_cast<GtkTreeIter*>(iter.get_gobject_if_not_end()));
+      const_cast<GtkTreeIter*>(iter.get_gobject_if_not_end())); //TODO: What does get_gobject_if_not_end() do?
 
-  if(new_pos.gobj()->stamp == 0)
-    new_pos.setup_end_iterator(iter);
+  if(new_pos.gobj()->stamp == 0) //TODO: What does stamp == 0 mean?
+    new_pos.setup_end_iterator(iter); //TODO: What does setup_end_iterator() do?
 
   return new_pos;
 }
@@ -106,38 +107,27 @@ namespace Gtk
 /* The *_Class implementation: */
 
 GType ListStore_Class::get_type()
+{
+  if(!gtype_) // create the GType if necessary
   {
-    if (!gtype_) //Create the GType if necessary.
-      {
-        //Make sure that the parent type has been created:
-        CppClassParent::CppObjectType::get_type();
+    // TODO: This is currently just optimized away, apparently with no harm.
+    // Is it actually needed?
+    // Make sure that the parent type has been created.
+    CppClassParent::CppObjectType::get_type();
 
-        //Create a derived C type.
-        //These init funcs will, in turn, call the init functions in the base C type.
-        GTypeInfo info =
-          {
-            sizeof(BaseClassType), /* class_size */
-            0, /* base_init */
-            0, /* base_finalize */
-            (GClassInitFunc) &class_init_function, /* class_init */
-            0, /* class_finalize */
-            0, /* class_data */
-            sizeof(BaseObjectType), /* instance_size */
-            0, /* n_preallocs */
-            0, /* instance_init */
-	    0 /* value_table */
-          };
+    // Create the wrapper type, with the same class/instance size as the base type.
+    register_derived_type(gtk_list_store_get_type(), (GClassInitFunc) &class_init_function);
 
-        gtype_ = g_type_register_static(gtk_list_store_get_type(), "gtkmm__ListStore", &info, GTypeFlags(0));
-
-  //Add derived versions of interfaces, if the C type implements any interfaces:
+    // Add derived versions of interfaces, if the C type implements any interfaces:
   TreeModel::add_interface(get_type());
   TreeSortable::add_interface(get_type());
   TreeDragSource::add_interface(get_type());
   TreeDragDest::add_interface(get_type());
-      }
-    return gtype_;
   }
+
+  return gtype_;
+}
+
 void ListStore_Class::class_init_function(BaseClassType* klass)
 {
   CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
