@@ -3,7 +3,7 @@
 #include <gtkmm/private/rc_p.h>
 
 // -*- c++ -*-
-/* $Id: rc.ccg,v 1.16 2002/04/09 17:08:13 daniel Exp $ */
+/* $Id: rc.ccg,v 1.17 2002/04/16 13:01:52 murrayc Exp $ */
 
 /* 
  *
@@ -25,13 +25,14 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <gtkmm/settings.h>
 #include <gtkmm/widget.h>
 #include <gtk/gtkrc.h>
 
 namespace Gtk
 {
 
-RC::RC(const Glib::ustring &filename)
+RC::RC(const Glib::ustring& filename)
 {
   //gtk_rc_init();
   gtk_rc_parse(filename.c_str());
@@ -44,14 +45,14 @@ RC::get_style(const Widget &widget)
 }
 
 void
-RC::parse_string(const Glib::ustring &rc_string)
+RC::parse_string(const Glib::ustring& rc_string)
 {
   gtk_rc_parse_string(const_cast<char*>(rc_string.c_str()));
 }
 
 
 void
-RC::add_default_file(const Glib::ustring &filename)
+RC::add_default_file(const Glib::ustring& filename)
 {
   gtk_rc_add_default_file(const_cast<char*>(filename.c_str()));
 }
@@ -71,19 +72,19 @@ RC::get_default_files()
 
 /* deprecated:
 void
-RC::add_widget_name_style(const Glib::RefPtr<const RcStyle>& style, const Glib::ustring &pattern)
+RC::add_widget_name_style(const Glib::RefPtr<const RcStyle>& style, const Glib::ustring& pattern)
 {
   gtk_rc_add_widget_name_style(const_cast<GtkRcStyle*>(style->gobj()), const_cast<char*>(pattern.c_str()));
 }
 
 void
-RC::add_widget_class_style(const Glib::RefPtr<const RcStyle>& style, const Glib::ustring &pattern)
+RC::add_widget_class_style(const Glib::RefPtr<const RcStyle>& style, const Glib::ustring& pattern)
 {
   gtk_rc_add_widget_class_style(const_cast<GtkRcStyle*>(style->gobj()), const_cast<char*>(pattern.c_str()));
 }
 
 void
-RC::add_class_style(const Glib::RefPtr<const RcStyle>& style, const Glib::ustring &pattern)
+RC::add_class_style(const Glib::RefPtr<const RcStyle>& style, const Glib::ustring& pattern)
 {
   gtk_rc_add_class_style(const_cast<GtkRcStyle*>(style->gobj()), const_cast<char*>(pattern.c_str()));
 }
@@ -97,16 +98,15 @@ RC::reparse_all()
 
 
 Glib::ustring
-RC::find_pixmap_in_path(GtkSettings* context, GScanner *scanner,
-			    const Glib::ustring &pixmap_file)
+RC::find_pixmap_in_path(const Glib::RefPtr<Settings>& context, GScanner *scanner, const Glib::ustring& pixmap_file)
 {
   return Glib::convert_return_gchar_ptr_to_ustring(
-      gtk_rc_find_pixmap_in_path(context, scanner, const_cast<char*>(pixmap_file.c_str())));
+      gtk_rc_find_pixmap_in_path(context->gobj(), scanner, const_cast<char*>(pixmap_file.c_str())));
 }
 
 
 Glib::ustring
-RC::find_module_in_path(const Glib::ustring &module_file)
+RC::find_module_in_path(const Glib::ustring& module_file)
 {
   return Glib::convert_return_gchar_ptr_to_ustring(
       gtk_rc_find_module_in_path(const_cast<char*>(module_file.c_str())));
@@ -151,34 +151,23 @@ namespace Gtk
 /* The *_Class implementation: */
 
 GType RcStyle_Class::get_type()
+{
+  if(!gtype_) // create the GType if necessary
   {
-    if (!gtype_) //Create the GType if necessary.
-      {
-        //Make sure that the parent type has been created:
-        CppClassParent::CppObjectType::get_type();
+    // TODO: This is currently just optimized away, apparently with no harm.
+    // Is it actually needed?
+    // Make sure that the parent type has been created.
+    CppClassParent::CppObjectType::get_type();
 
-        //Create a derived C type.
-        //These init funcs will, in turn, call the init functions in the base C type.
-        GTypeInfo info =
-          {
-            sizeof(BaseClassType), /* class_size */
-            0, /* base_init */
-            0, /* base_finalize */
-            (GClassInitFunc) &class_init_function, /* class_init */
-            0, /* class_finalize */
-            0, /* class_data */
-            sizeof(BaseObjectType), /* instance_size */
-            0, /* n_preallocs */
-            0, /* instance_init */
-	    0 /* value_table */
-          };
+    // Create the wrapper type, with the same class/instance size as the base type.
+    register_derived_type(gtk_rc_style_get_type(), (GClassInitFunc) &class_init_function);
 
-        gtype_ = g_type_register_static(gtk_rc_style_get_type(), "gtkmm__RcStyle", &info, GTypeFlags(0));
-
-  //Add derived versions of interfaces, if the C type implements any interfaces:
-      }
-    return gtype_;
+    // Add derived versions of interfaces, if the C type implements any interfaces:
   }
+
+  return gtype_;
+}
+
 void RcStyle_Class::class_init_function(BaseClassType* klass)
 {
   CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
