@@ -3,7 +3,7 @@
 #include <gtkmm/private/statusbar_p.h>
 
 // -*- c++ -*-
-/* $Id: statusbar.ccg,v 1.3 2002/01/23 01:17:35 murrayc Exp $ */
+/* $Id: statusbar.ccg,v 1.5 2002/04/21 23:37:06 murrayc Exp $ */
 
 /* 
  *
@@ -25,16 +25,17 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <gtk/gtkstatusbar.h>
 
 namespace Gtk
 {
 
-guint Statusbar::push(const Glib::ustring& text, guint context_id = 0)
+guint Statusbar::push(const Glib::ustring& text, guint context_id /* = 0 */)
 {
   return gtk_statusbar_push(gobj(), context_id, text.c_str());
 }
 
-void Statusbar::remove_message(guint message_id, guint context_id = 0)
+void Statusbar::remove_message(guint message_id, guint context_id /* = 0 */)
 {
   return gtk_statusbar_remove(gobj(), context_id, message_id);
 }
@@ -117,34 +118,23 @@ namespace Gtk
 /* The *_Class implementation: */
 
 GType Statusbar_Class::get_type()
+{
+  if(!gtype_) // create the GType if necessary
   {
-    if (!gtype_) //Create the GType if necessary.
-      {
-        //Make sure that the parent type has been created:
-        CppClassParent::CppObjectType::get_type();
+    // TODO: This is currently just optimized away, apparently with no harm.
+    // Is it actually needed?
+    // Make sure that the parent type has been created.
+    CppClassParent::CppObjectType::get_type();
 
-        //Create a derived C type.
-        //These init funcs will, in turn, call the init functions in the base C type.
-        GTypeInfo info =
-          {
-            sizeof(BaseClassType), /* class_size */
-            0, /* base_init */
-            0, /* base_finalize */
-            (GClassInitFunc) &class_init_function, /* class_init */
-            0, /* class_finalize */
-            0, /* class_data */
-            sizeof(BaseObjectType), /* instance_size */
-            0, /* n_preallocs */
-            0, /* instance_init */
-	    0 /* value_table */
-          };
+    // Create the wrapper type, with the same class/instance size as the base type.
+    register_derived_type(gtk_statusbar_get_type(), (GClassInitFunc) &class_init_function);
 
-        gtype_ = g_type_register_static(gtk_statusbar_get_type(), "gtkmm__Statusbar", &info, GTypeFlags(0));
-
-  //Add derived versions of interfaces, if the C type implements any interfaces:
-      }
-    return gtype_;
+    // Add derived versions of interfaces, if the C type implements any interfaces:
   }
+
+  return gtype_;
+}
+
 void Statusbar_Class::class_init_function(BaseClassType* klass)
 {
   CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
