@@ -3,7 +3,7 @@
 #include <gtkmm/private/textbuffer_p.h>
 
 // -*- c++ -*-
-/* $Id: textbuffer.ccg,v 1.17 2002/03/26 18:43:28 daniel Exp $ */
+/* $Id: textbuffer.ccg,v 1.18 2002/04/16 13:01:53 murrayc Exp $ */
 
 /* 
  *
@@ -25,9 +25,11 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <gtk/gtktextbuffer.h>
 #include <glib/gslist.h>
 #include <gtkmm/textmark.h>
 
+
 namespace Gtk
 {
 
@@ -543,34 +545,23 @@ namespace Gtk
 /* The *_Class implementation: */
 
 GType TextBuffer_Class::get_type()
+{
+  if(!gtype_) // create the GType if necessary
   {
-    if (!gtype_) //Create the GType if necessary.
-      {
-        //Make sure that the parent type has been created:
-        CppClassParent::CppObjectType::get_type();
+    // TODO: This is currently just optimized away, apparently with no harm.
+    // Is it actually needed?
+    // Make sure that the parent type has been created.
+    CppClassParent::CppObjectType::get_type();
 
-        //Create a derived C type.
-        //These init funcs will, in turn, call the init functions in the base C type.
-        GTypeInfo info =
-          {
-            sizeof(BaseClassType), /* class_size */
-            0, /* base_init */
-            0, /* base_finalize */
-            (GClassInitFunc) &class_init_function, /* class_init */
-            0, /* class_finalize */
-            0, /* class_data */
-            sizeof(BaseObjectType), /* instance_size */
-            0, /* n_preallocs */
-            0, /* instance_init */
-	    0 /* value_table */
-          };
+    // Create the wrapper type, with the same class/instance size as the base type.
+    register_derived_type(gtk_text_buffer_get_type(), (GClassInitFunc) &class_init_function);
 
-        gtype_ = g_type_register_static(gtk_text_buffer_get_type(), "gtkmm__TextBuffer", &info, GTypeFlags(0));
-
-  //Add derived versions of interfaces, if the C type implements any interfaces:
-      }
-    return gtype_;
+    // Add derived versions of interfaces, if the C type implements any interfaces:
   }
+
+  return gtype_;
+}
+
 void TextBuffer_Class::class_init_function(BaseClassType* klass)
 {
   CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
