@@ -4,7 +4,7 @@
 
 #include <gtk/gtktypebuiltins.h>
 // -*- c++ -*-
-/* $Id: treemodel.ccg,v 1.9 2002/04/02 13:25:19 daniel Exp $ */
+/* $Id: treemodel.ccg,v 1.10 2002/04/16 13:01:53 murrayc Exp $ */
 
 /* 
  *
@@ -26,6 +26,7 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <gtk/gtktreestore.h>
 #include <gtk/gtktreemodel.h>
 
 namespace Gtk
@@ -33,7 +34,7 @@ namespace Gtk
 
 TreeIter TreeModel::get_iter(Gtk::TreePath& path)
 {
-  TreeIter iter (gobj());
+  TreeIter iter(gobj());
   gtk_tree_model_get_iter(gobj(), iter.gobj(), path.gobj());
   return iter;
 }
