@@ -40,11 +40,12 @@ class TreeModelColumnBase;
  * @ingroup TreeView
  * TreeModelColumnRecord objects are used to setup a new TreeModel
  * (or rather, an implementation of the model, like Gtk::ListStore
- * or Gtk::TreeStore).  A convenient way to do that is deriving from
+ * or Gtk::TreeStore).  A convenient way to do that is by deriving from
  * TreeModelColumnRecord:
  * @code
- * struct MyModelColumns : public Gtk::TreeModelColumnRecord
+ * class MyModelColumns : public Gtk::TreeModelColumnRecord
  * {
+ * public:
  *   Gtk::TreeModelColumn<Glib::ustring>                filename;
  *   Gtk::TreeModelColumn<Glib::ustring>                description;
  *   Gtk::TreeModelColumn< Glib::RefPtr<Gdk::Pixbuf> >  thumbnail;
@@ -53,14 +54,14 @@ class TreeModelColumnBase;
  * };
  * @endcode
  * An instance of @c MyModelColumns should be passed to ListStore::create()
- * or TreeStore::create().  The struct members @c filename, @c description
+ * or TreeStore::create().  The members @c filename, @c description
  * and @c thumbnail can then be used with
  * @link Gtk::TreeModel::get_value() get_value@endlink() and
  * @link Gtk::TreeStore::set_value() set_value@endlink(),
  * in order to specify the column you're interested in.
  *
  * Neither TreeModelColumnRecord nor the
- * @link TreeModelColumn TreeModelColumns@endlink contain any real data,
+ * @link TreeModelColumn TreeModelColumns@endlink contain any real data -
  * it's merely a description of what type is stored in which column
  * of a TreeModel.  Thus it's absolutely legal to use a statically allocated
  * TreeModelColumnRecord (as long as you make sure it's instantiated after
@@ -70,7 +71,7 @@ class TreeModelColumnRecord
 {
 public:
   TreeModelColumnRecord();
-  ~TreeModelColumnRecord();
+  ~TreeModelColumnRecord(); //TODO: Either a) make it virtual or b) Explain why it should not be virtual (It's already non-copyable)
 
   /** Adds a TreeModelColumn to this record.
    * add() not only registers the @a column, but also assigns a column
@@ -82,7 +83,7 @@ public:
   unsigned int size()  const;
   const GType* types() const;
 
-private:
+protected:
   std::vector<GType> column_types_;
 
   // noncopyable
@@ -103,7 +104,7 @@ public:
 protected:
   explicit TreeModelColumnBase(GType type);
 
-private:
+protected:
   GType type_;
   int   index_;
 
