@@ -4,7 +4,7 @@
 
 #include <gtk/gtktypebuiltins.h>
 // -*- c++ -*-
-/* $Id: treeview.ccg,v 1.10 2002/04/02 13:25:20 daniel Exp $ */
+/* $Id: treeview.ccg,v 1.12 2002/04/18 05:52:20 murrayc Exp $ */
 
 /* 
  *
@@ -33,6 +33,147 @@
 namespace Gtk
 {
 
+//SignalProxy_Mapping:
+
+//This Signal Proxy allows the C++ coder to specify a SigC::Slot instead of a static function.
+class SignalProxy_Mapping
+{
+public:
+  typedef TreeView::SlotMapping SlotType;
+
+  SignalProxy_Mapping(const SlotType& slot);
+  ~SignalProxy_Mapping();
+
+  static void gtk_callback(GtkTreeView* tree_view, GtkTreePath* path, gpointer data);
+
+protected:
+  SlotType slot_;
+};
+
+SignalProxy_Mapping::SignalProxy_Mapping(const SlotType& slot)
+{
+  slot_ = slot;
+}
+
+SignalProxy_Mapping::~SignalProxy_Mapping()
+{
+}
+
+void SignalProxy_Mapping::gtk_callback(GtkTreeView* tree_view, GtkTreePath* path, gpointer data)
+{
+  SignalProxy_Mapping* pSignalProxy = static_cast<SignalProxy_Mapping*>(data);
+  if(pSignalProxy)
+  {
+    //Use Slot::operator():
+    TreeView* pCppTreeView = Glib::wrap(tree_view);
+    TreePath cppTreePath = Glib::wrap(path, true); //true = copy
+    (pSignalProxy->slot_)(pCppTreeView, cppTreePath);
+  }
+}
+
+
+//SignalProxy_SearchEqual:
+
+//This Signal Proxy allows the C++ coder to specify a SigC::Slot instead of a static function.
+class SignalProxy_SearchEqual
+{
+public:
+  typedef TreeView::SlotSearchEqual SlotType;
+
+  SignalProxy_SearchEqual(const SlotType& slot);
+  ~SignalProxy_SearchEqual();
+
+  static gboolean gtk_callback(GtkTreeModel* model, gint column, const gchar* key, GtkTreeIter* iter, gpointer data);
+  static void gtk_callback_destroy(gpointer data);
+
+protected:
+  SlotType slot_;
+};
+
+SignalProxy_SearchEqual::SignalProxy_SearchEqual(const SlotType& slot)
+{
+  slot_ = slot;
+}
+
+SignalProxy_SearchEqual::~SignalProxy_SearchEqual()
+{
+}
+
+gboolean SignalProxy_SearchEqual::gtk_callback(GtkTreeModel* model, gint column, const gchar* key, GtkTreeIter* iter, gpointer data)
+{
+  SignalProxy_SearchEqual* pSignalProxy = static_cast<SignalProxy_SearchEqual*>(data);
+  if(pSignalProxy)
+  {
+    //Use Slot::operator():
+    Glib::RefPtr<TreeModel> refModel = Glib::wrap(model, true); //true = copy.
+    TreeIter cppTreeIter = TreeIter(model, iter); //Seems to be the standard conversion, specified in an m4 macro in treeview.hg.
+    return (gboolean)(pSignalProxy->slot_)(refModel, column, key, cppTreeIter);
+  }
+
+  return false; //arbitrary value.
+}
+
+void SignalProxy_SearchEqual::gtk_callback_destroy(gpointer data)
+{
+  SignalProxy_SearchEqual* pSignalProxy = static_cast<SignalProxy_SearchEqual*>(data);
+  if(pSignalProxy)
+    delete pSignalProxy;
+}
+
+
+//SignalProxy_ColumnDrop:
+
+//This Signal Proxy allows the C++ coder to specify a SigC::Slot instead of a static function.
+class SignalProxy_ColumnDrop
+{
+public:
+  typedef TreeView::SlotColumnDrop SlotType;
+
+  SignalProxy_ColumnDrop(const SlotType& slot);
+  ~SignalProxy_ColumnDrop();
+
+  static gboolean gtk_callback(GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* prev_column, GtkTreeViewColumn* next_column, gpointer data);
+  static void gtk_callback_destroy(gpointer data);
+
+protected:
+  SlotType slot_;
+};
+
+SignalProxy_ColumnDrop::SignalProxy_ColumnDrop(const SlotType& slot)
+{
+  slot_ = slot;
+}
+
+SignalProxy_ColumnDrop::~SignalProxy_ColumnDrop()
+{
+}
+
+gboolean SignalProxy_ColumnDrop::gtk_callback(GtkTreeView* tree_view, GtkTreeViewColumn* column, GtkTreeViewColumn* prev_column, GtkTreeViewColumn* next_column, gpointer data)
+{
+  SignalProxy_ColumnDrop* pSignalProxy = static_cast<SignalProxy_ColumnDrop*>(data);
+  if(pSignalProxy)
+  {
+    //Use Slot::operator():
+    TreeView* pCppTreeView = Glib::wrap(tree_view);
+    TreeViewColumn* pCppColumn = Glib::wrap(column);
+    TreeViewColumn* pCppColumnPrevious = Glib::wrap(column);
+    TreeViewColumn* pCppColumnNext = Glib::wrap(column);
+
+    return (gboolean)(pSignalProxy->slot_)(pCppTreeView, pCppColumn, pCppColumnPrevious, pCppColumnNext);
+  }
+
+  return false; //arbitrary value.
+}
+
+void SignalProxy_ColumnDrop::gtk_callback_destroy(gpointer data)
+{
+  SignalProxy_ColumnDrop* pSignalProxy = static_cast<SignalProxy_ColumnDrop*>(data);
+  if(pSignalProxy)
+    delete pSignalProxy;
+}
+
+
+//TreeView implementation:
 
 int TreeView::insert_column_with_data_func(int position, const Glib::ustring& title, CellRenderer& cell, const SlotCellData& slot)
 {
@@ -90,6 +231,38 @@ bool TreeView::get_dest_row_at_pos(int drag_x, int drag_y, TreePath& path, TreeV
   return bResult;
 }
 
+void TreeView::map_expanded_rows(const SlotMapping slot)
+{
+  //Create a signal proxy.
+  SignalProxy_Mapping signalProxy(slot); //It only needs to live as long as this method call.
+
+  gtk_tree_view_map_expanded_rows( gobj(), &SignalProxy_Mapping::gtk_callback, (gpointer)(&signalProxy) );
+}
+
+void TreeView::set_search_equal_func(const SlotSearchEqual& slot)
+{
+  //Create a signal proxy. A pointer to this will be passed through the callback's data parameter.
+  //It will be deleted when SignalProxy_SearchEqual::gtk_callback_destroy() is called.
+  SignalProxy_SearchEqual* pSignalProxy = new SignalProxy_SearchEqual(slot);
+
+  gtk_tree_view_set_search_equal_func(gobj(),
+    &SignalProxy_SearchEqual::gtk_callback, (gpointer)pSignalProxy,
+    &SignalProxy_SearchEqual::gtk_callback_destroy);
+}
+
+void TreeView::set_column_drag_function(const SlotColumnDrop& slot)
+{
+  //Create a signal proxy. A pointer to this will be passed through the callback's data parameter.
+  //It will be deleted when SignalProxy_ColumnDrop::gtk_callback_destroy() is called.
+  SignalProxy_ColumnDrop* pSignalProxy = new SignalProxy_ColumnDrop(slot);
+
+  gtk_tree_view_set_column_drag_function(gobj(),
+    &SignalProxy_ColumnDrop::gtk_callback, (gpointer)pSignalProxy,
+    &SignalProxy_ColumnDrop::gtk_callback_destroy);
+
+}
+
+
 } /* namespace Gtk */
 
 
@@ -388,6 +561,14 @@ const Glib::SignalProxyInfo TreeView_signal_select_all_info =
 };
 
 
+const Glib::SignalProxyInfo TreeView_signal_unselect_all_info =
+{
+  "unselect-all",
+  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback,
+  (GCallback) &Glib::SignalProxyNormal::slot0_void_callback
+};
+
+
 void TreeView_signal_select_cursor_row_callback(GtkTreeView*, gboolean p0,void* data)
 {
   using namespace Gtk;
@@ -491,34 +672,23 @@ namespace Gtk
 /* The *_Class implementation: */
 
 GType TreeView_Class::get_type()
+{
+  if(!gtype_) // create the GType if necessary
   {
-    if (!gtype_) //Create the GType if necessary.
-      {
-        //Make sure that the parent type has been created:
-        CppClassParent::CppObjectType::get_type();
+    // TODO: This is currently just optimized away, apparently with no harm.
+    // Is it actually needed?
+    // Make sure that the parent type has been created.
+    CppClassParent::CppObjectType::get_type();
 
-        //Create a derived C type.
-        //These init funcs will, in turn, call the init functions in the base C type.
-        GTypeInfo info =
-          {
-            sizeof(BaseClassType), /* class_size */
-            0, /* base_init */
-            0, /* base_finalize */
-            (GClassInitFunc) &class_init_function, /* class_init */
-            0, /* class_finalize */
-            0, /* class_data */
-            sizeof(BaseObjectType), /* instance_size */
-            0, /* n_preallocs */
-            0, /* instance_init */
-	    0 /* value_table */
-          };
+    // Create the wrapper type, with the same class/instance size as the base type.
+    register_derived_type(gtk_tree_view_get_type(), (GClassInitFunc) &class_init_function);
 
-        gtype_ = g_type_register_static(gtk_tree_view_get_type(), "gtkmm__TreeView", &info, GTypeFlags(0));
-
-  //Add derived versions of interfaces, if the C type implements any interfaces:
-      }
-    return gtype_;
+    // Add derived versions of interfaces, if the C type implements any interfaces:
   }
+
+  return gtype_;
+}
+
 void TreeView_Class::class_init_function(BaseClassType* klass)
 {
   CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
@@ -532,6 +702,7 @@ void TreeView_Class::class_init_function(BaseClassType* klass)
   klass->columns_changed = &columns_changed_callback;
   klass->move_cursor = &move_cursor_callback;
   klass->select_all = &select_all_callback;
+  klass->unselect_all = &unselect_all_callback;
   klass->select_cursor_row = &select_cursor_row_callback;
   klass->toggle_cursor_row = &toggle_cursor_row_callback;
   klass->expand_collapse_cursor_row = &expand_collapse_cursor_row_callback;
@@ -843,6 +1014,34 @@ void TreeView_Class::select_all_callback(GtkTreeView* self)
   }
 }
 
+void TreeView_Class::unselect_all_callback(GtkTreeView* self)
+{
+  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
+      Glib::ObjectBase::_get_current_wrapper((GObject*)self));
+
+  if(obj)
+  {
+    try
+    {
+      obj->on_unselect_all();
+    }
+    catch(...)
+    {
+      Glib::exception_handlers_invoke();
+    }
+  }
+  else
+  {
+    BaseClassType *const base = static_cast<BaseClassType*>(
+        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
+);
+    g_assert(base != 0);
+
+    if(base->unselect_all)
+      (*base->unselect_all)(self);
+  }
+}
+
 void TreeView_Class::select_cursor_row_callback(GtkTreeView* self, gboolean p0)
 {
   CppObjectType *const obj = dynamic_cast<CppObjectType*>(
@@ -1136,11 +1335,6 @@ TreeViewColumn* TreeView::get_expander_column()
   return Glib::wrap(gtk_tree_view_get_expander_column(gobj()));
 }
 
-void TreeView::set_column_drag_function(GtkTreeViewColumnDropFunc func, gpointer user_data, GtkDestroyNotify destroy)
-{
-  gtk_tree_view_set_column_drag_function(gobj(), func, user_data, destroy);
-}
-
 void TreeView::scroll_to_point(int tree_x, int tree_y)
 {
   gtk_tree_view_scroll_to_point(gobj(), tree_x, tree_y);
@@ -1176,11 +1370,6 @@ bool TreeView::collapse_row(const TreePath& path)
   return gtk_tree_view_collapse_row(gobj(), const_cast<GtkTreePath*>((path).gobj()));
 }
 
-void TreeView::map_expanded_rows(GtkTreeViewMappingFunc func, gpointer data)
-{
-  gtk_tree_view_map_expanded_rows(gobj(), func, data);
-}
-
 bool TreeView::row_expanded(const TreePath& path)
 {
   return gtk_tree_view_row_expanded(gobj(), const_cast<GtkTreePath*>((path).gobj()));
@@ -1284,16 +1473,6 @@ void TreeView::set_search_column(int column)
   gtk_tree_view_set_search_column(gobj(), column);
 }
 
-GtkTreeViewSearchEqualFunc TreeView::get_search_equal_func()
-{
-  return gtk_tree_view_get_search_equal_func(gobj());
-}
-
-void TreeView::set_search_equal_func(GtkTreeViewSearchEqualFunc search_equal_func, gpointer search_user_data, GtkDestroyNotify search_destroy)
-{
-  gtk_tree_view_set_search_equal_func(gobj(), search_equal_func, search_user_data, search_destroy);
-}
-
 
 Glib::SignalProxy2<void,Adjustment*,Adjustment*> TreeView::signal_set_scroll_adjustments()
 {
@@ -1345,6 +1524,11 @@ Glib::SignalProxy0<void> TreeView::signal_select_all()
   return Glib::SignalProxy0<void>(this, &TreeView_signal_select_all_info);
 }
 
+Glib::SignalProxy0<void> TreeView::signal_unselect_all()
+{
+  return Glib::SignalProxy0<void>(this, &TreeView_signal_unselect_all_info);
+}
+
 Glib::SignalProxy1<void,bool> TreeView::signal_select_cursor_row()
 {
   return Glib::SignalProxy1<void,bool>(this, &TreeView_signal_select_cursor_row_info);
@@ -1541,6 +1725,17 @@ void Gtk::TreeView::on_select_all()
     (*base->select_all)(gobj());
 }
 
+void Gtk::TreeView::on_unselect_all()
+{
+  BaseClassType *const base = static_cast<BaseClassType*>(
+      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
+);
+  g_assert(base != 0);
+
+  if(base->unselect_all)
+    (*base->unselect_all)(gobj());
+}
+
 void Gtk::TreeView::on_select_cursor_row(bool start_editing)
 {
   BaseClassType *const base = static_cast<BaseClassType*>(
