@@ -65,11 +65,18 @@ namespace Pango
 /* The *_Class implementation: */
 
 GType Font_Class::get_type()
+{
+  if(!gtype_) // create the GType if necessary
   {
-    gtype_ = pango_font_get_type(); //The actual struct definition is hidden, so we can not derive one.
-    return gtype_;
+    // The actual struct definition is hidden, so we can not derive one.
+    gtype_ = pango_font_get_type();
+
+    // Add derived versions of interfaces, if the C type implements any interfaces:
   }
 
+  return gtype_;
+}
+
 
 Glib::ObjectBase* Font_Class::wrap_new(GObject* o)
 {
