@@ -350,13 +350,14 @@ sub output_wrap_ctor($, $, $, $, $)
 
 sub output_wrap_create($,$,$)
 {
-  my ($self, $args_type_and_name, $objWrapParser) = @_;
+  my ($self, $args_type_and_name_with_default_values, $objWrapParser) = @_;
 
   #Re-use Function in a very hacky way, to separate the argument types_and_names.
-  my $fake_decl = "void fake_func(" . $args_type_and_name . ")";
+  my $fake_decl = "void fake_func(" . $args_type_and_name_with_default_values . ")";
 
   my $objFunction = &Function::new($fake_decl, $objWrapParser);
   my $args_names_only = $objFunction->args_names_only();
+  my $args_type_and_name = $objFunction->args_types_and_names();
 
   my $str = sprintf("_CREATE_METHOD(\`%s\',\`%s\')dnl\n",
               $args_type_and_name, $args_names_only);
@@ -404,12 +405,10 @@ sub output_wrap_sig_decl($, $, $, $, $, $)
   $self->append($str);
 }
 
-# void output_wrap_enum($filename, $line_num, $cpp_type, $c_type, $comment)
-sub output_wrap_enum($, $, $, $, $, $)
+# void output_wrap_enum($filename, $line_num, $cpp_type, $c_type, $no_gtype, $comment)
+sub output_wrap_enum($, $, $, $, $, $, $)
 {
-  my ($self, $filename, $line_num, $cpp_type, $c_type, $comment) = @_;
-
-  my $objDefsParser = $$self{objDefsParser};
+  my ($self, $filename, $line_num, $cpp_type, $c_type, $flags, $comment) = @_;
 
   my $objEnum = GtkDefs::lookup_enum($c_type);
   if(!$objEnum)
@@ -439,17 +438,72 @@ sub output_wrap_enum($, $, $, $, $, $)
   my $value_suffix = "Enum";
   $value_suffix = "Flags" if($$objEnum{flags});
 
-  my $str = sprintf("_ENUM(%s,%s,%s,\`%s\',\`%s\')dnl\n",
+  my $str = sprintf("_ENUM(%s,%s,%s,\`%s\',\`%s\',\`%s\')dnl\n",
     $cpp_type,
     $c_type,
     $value_suffix,
     $elements,
+    $flags,
     $comment
   );
 
   $self->append($str);
 }
 
+# void output_wrap_gerror($filename, $line_num, $cpp_type, $c_enum, $domain, $no_gtype)
+sub output_wrap_gerror($, $, $, $, $, $, $)
+{
+  my ($self, $filename, $line_num, $cpp_type, $c_enum, $domain, $no_gtype) = @_;
+
+  my $objDefsParser = $$self{objDefsParser};
+
+  my $objEnum = GtkDefs::lookup_enum($c_enum);
+  if(!$objEnum)
+  {
+    $self->output_wrap_failed($c_enum, "enum defs lookup failed.");
+    return;
+  }
+
+  # Shouldn't happen, and if it does, I'd like to know that.
+  warn if($$objEnum{flags});
+
+  # cut off the module prefix, e.g. GDK_
+  $domain =~ s/^[^_]+_//;
+
+  $objEnum->beautify_values();
+
+  my $elem_names  = $$objEnum{elem_names};
+  my $elem_values = $$objEnum{elem_values};
+
+  my $num_elements = scalar(@{$elem_names});
+  my $elements = "";
+
+  for(my $i = 0; $i < $num_elements; ++$i)
+  {
+    my $name  = $$elem_names[$i];
+    my $value = $$elem_values[$i];
+
+    # Chop off the domain prefix, because we put the enum into the class.
+    $name  =~ s/^${domain}_//;
+    $value =~ s/^${domain}_//; # handle aliases
+
+    if($name eq "2BIG") { $name = "TOOBIG"; } # hack for GSpawnError
+
+    $elements .= "    ${name}";
+    $elements .= " = ${value}" if($value ne "");
+    $elements .= ",\n" if($i < $num_elements - 1);
+  }
+
+  my $str = sprintf("_GERROR(%s,%s,\`%s\',%s)dnl\n",
+    $cpp_type,
+    $c_enum,
+    $elements,
+    $no_gtype
+  );
+
+  $self->append($str);
+}
+
 # _PROPERTY_PROXY(name, cpp_type)
 # void output_wrap_property($filename, $line_num, $name, $cpp_type)
 sub output_wrap_property($, $, $, $, $, $)
@@ -766,5 +820,22 @@ sub get_ctor_properties($,$,$,$)
   return join(", ", @result);
 }
 
+### Convert _WRAP to a corba method
+# _CORBA_METHOD(retype, method_name,args, arg_names_only) - implemented in libbonobomm.
+#  void output_wrap_corba_method($filename, $line_num, $objCppFunc)
+sub output_wrap_corba_method($, $, $, $)
+{
+  my ($self, $filename, $line_num, $objCppfunc) = @_;
+
+  my $str = sprintf("_CORBA_METHOD(%s,%s,\`%s\',\`%s\')dnl\n",
+      $$objCppfunc{rettype},
+      $$objCppfunc{name},
+      $objCppfunc->args_types_and_names(),
+      $objCppfunc->args_names_only()
+   );
+
+  $self->append($str);
+}
+
 
 1; # indicate proper module load.
