@@ -31,11 +31,11 @@ BEGIN {
      # set the version for version checking
      $VERSION     = 1.00;
      @ISA         = qw(Exporter);
-     @EXPORT      = qw(&func1 &func2 &func4);
+     @EXPORT      = ( );
      %EXPORT_TAGS = ( );
      # your exported package globals go here,
      # as well as any optionally exported functions
-     @EXPORT_OK   = qw($Var1 %Hashit &func3);
+     @EXPORT_OK   = ( );
      }
 our @EXPORT_OK;
 
@@ -110,6 +110,7 @@ sub parse_and_build_output($)
     if ($token eq "_IGNORE")     { $self->on_ignore(); next;} #Ignore a function.
     if ($token eq "_IGNORE_SIGNAL")     { $self->on_ignore_signal(); next;}
     if ($token eq "_WRAP_METHOD")     { $self->on_wrap_method(); next;}
+    if ($token eq "_WRAP_CORBA_METHOD")     { $self->on_wrap_corba_method(); next;} #Used in libbonobo*mm.
     if ($token eq "_WRAP_SIGNAL") { $self->on_wrap_signal(); next;}
     if ($token eq "_WRAP_PROPERTY") { $self->on_wrap_property(); next;}
     if ($token eq "_WRAP_VFUNC") { $self->on_wrap_vfunc(0); next;} # bVfuncOnly=0
@@ -118,15 +119,25 @@ sub parse_and_build_output($)
     if ($token eq "_WRAP_CREATE") { $self->on_wrap_create(); next;}
 
     if ($token eq "_WRAP_ENUM")   { $self->on_wrap_enum(); next;}
+    if ($token eq "_WRAP_GERROR") { $self->on_wrap_gerror(); next;}
 
-    if ($token eq "_CLASS_GENERIC")           { $self->on_class($token); next;}
-    if ($token eq "_CLASS_GOBJECT")           { $self->on_class($token); next;}
-    if ($token eq "_CLASS_GTKOBJECT")         { $self->on_class($token); next;}
-    if ($token eq "_CLASS_BOXEDTYPE")         { $self->on_class($token); next;}
-    if ($token eq "_CLASS_BOXEDTYPE_STATIC")  { $self->on_class($token); next;}
-    if ($token eq "_CLASS_INTERFACE")         { $self->on_class($token); next;}
-    if ($token eq "_CLASS_OPAQUE_COPYABLE")   { $self->on_class($token); next;}
-    if ($token eq "_CLASS_OPAQUE_REFCOUNTED") { $self->on_class($token); next;}
+    my $prefix_class = "_CLASS_"; # e.g. _CLASS_GTKOBJECT
+    my $token_prefix = substr($token, 0, length($prefix_class));
+    if ($token_prefix eq $prefix_class)
+    {
+      $self->on_class($token);
+      next;
+
+      # e.g.:
+      # _CLASS_GENERIC
+      # _CLASS_GOBJECT
+      # _CLASS_GTKOBJECT
+      # _CLASS_BOXEDTYPE
+      # _CLASS_BOXEDTYPE_STATIC
+      # _CLASS_INTERFACE
+      # _CLASS_OPAQUE_COPYABLE
+      # _CLASS_OPAQUE_REFCOUNTED
+    }
 
     if ($token eq "namespace") { $self->on_namespace() };
 
@@ -910,12 +921,13 @@ sub on_wrap_enum($)
   {
     # steal the last two tokens
     my @back = splice(@$out, -2);
+    local $_ = $back[0];
 
-    # Check for /** ... */ or /// comments.  The closing */ _must_ be
-    # the last token of the previous line.  Apart from this restriction,
+    # Check for /*[*!] ... */ or //[/!] comments.  The closing */ _must_
+    # be the last token of the previous line.  Apart from this restriction,
     # anything else should work, including multi-line comments.
 
-    if($back[1] eq "\n" && ($back[0] =~ m#^/\*`\*(.+)'\*/#s || $back[0] =~ m#^//`/(.+)'$#))
+    if($back[1] eq "\n" && (m#^/\*`[*!](.+)'\*/#s || m#^//`[/!](.+)'$#))
     {
       $comment = $1;
       $comment =~ s/\s+$//;
@@ -933,7 +945,31 @@ sub on_wrap_enum($)
   my $cpp_type = string_trim($args[0]);
   my $c_type   = string_trim($args[1]);
 
-  $outputter->output_wrap_enum($$self{filename}, $$self{line_num}, $cpp_type, $c_type, $comment);
+  my $flags = "";
+  $flags = string_trim($args[2]) if(scalar(@args) > 2);
+
+  $outputter->output_wrap_enum(
+      $$self{filename}, $$self{line_num}, $cpp_type, $c_type, $flags, $comment);
+}
+
+sub on_wrap_gerror($)
+{
+  my ($self) = @_;
+
+  return if(!$self->check_for_eof());
+
+  # get the arguments
+  my @args = string_split_commas($self->extract_bracketed_text());
+
+  my $cpp_type = string_trim($args[0]);
+  my $c_enum   = string_trim($args[1]);
+  my $domain   = string_trim($args[2]);
+
+  my $no_gtype = "";
+  $no_gtype = string_trim($args[3]) if(scalar(@args) > 3);
+
+  $$self{objOutputter}->output_wrap_gerror(
+      $$self{filename}, $$self{line_num}, $cpp_type, $c_enum, $domain, $no_gtype);
 }
 
 sub on_wrap_property($)
@@ -1102,4 +1138,50 @@ sub byattrib()
   return $attrib_value{$b} <=> $attrib_value{$a}; 
 }
 
+
+# void on_wrap_corba_method()
+sub on_wrap_corba_method($)
+{
+  my ($self) = @_;
+  my $objOutputter = $$self{objOutputter};
+
+  if( !($self->check_for_eof()) )
+  {
+   return;
+  }
+
+  my $filename = $$self{filename};
+  my $line_num = $$self{line_num};
+
+  my $str = $self->extract_bracketed_text();
+  my @args = string_split_commas($str);
+
+  my $entity_type = "method";
+
+  if (!$$self{in_class})
+    {
+      print STDERR "$filename:$line_num:_WRAP macro encountered outside class\n";
+      return;
+    }
+
+  my $objCfunc;
+  my $objCppfunc;
+
+  # handle first argument
+  my $argCppMethodDecl = $args[0];
+  if ($argCppMethodDecl =~ /^\S+$/ ) #Checks that it's not empty and that it contains no whitespace.
+  {
+    print STDERR "$filename:$line_num:_WRAP can't handle unspecified method $argCppMethodDecl\n";
+    return;
+  }
+  else
+  {
+    #Parse the method decaration and build an object that holds the details:
+    $objCppfunc = &Function::new($argCppMethodDecl, $self);
+  }
+
+  $objOutputter->output_wrap_corba_method($filename, $line_num, $objCppfunc);
+}
+
+
 1; # return package loaded okay.
